from django.core.management.base import BaseCommand, CommandError
import django_configure


class Command(BaseCommand):
    help = 'Generates config file and wsgi file in designated location'

    def add_arguments(self, parser):
        parser.add_argument('config_path', type=str)
        parser.add_argument('wsgi_path', nargs='?', type=str)
        parser.add_argument('--dev', action='store_true')

    def handle(self, *args, **options):
        config = django_configure.main_config

        config.config_path = options['config_path']
        config_output = config.generate(dev=options['dev'])

        self.stdout.write(self.style.SUCCESS('Successfully generated "%s"' % options['config_path']))
        self.stdout.write('Config output:')
        self.stdout.write(config_output)

        if options['wsgi_path'] is not None:
            wsgi_output = config.generate_wsgi(options['wsgi_path'])
            self.stdout.write(self.style.SUCCESS('Successfully generated "%s"' % options['wsgi_path']))
            self.stdout.write('Wsgi output:')
            self.stdout.write(wsgi_output)



# coding=utf-8
from __future__ import unicode_literals
import sys

import os

from django.utils.crypto import get_random_string

import django_configure.type
import ConfigParser

main_config = None


class ConfigParserDefault(ConfigParser.ConfigParser):
    def get(self, section, option, *args, **kwargs):
        try:
            return ConfigParser.ConfigParser.get(self, section, option)
        except Exception:
            return None


class DjangoConfig(object):
    def __init__(self, description_dict={}, env_prefix='', path=None, virtual_env_root=True):
        self.content = description_dict
        self.env_prefix = env_prefix
        self.virtual_env_root = virtual_env_root
        self.config_path = path
        self.config = ConfigParserDefault()

        if self.config_var_name in os.environ:
            self.config.read(os.environ[self.config_var_name])
        # we are not in virtualenv, or we explicitly do not want config path relative to virtual env
        if self.config_path is not None and os.path.exists(self.config_path):
            if not self.virtual_env_root and hasattr(sys, 'real_prefix'):
                self.config.read(self.config_path)
            else:
                # we are in virtual env
                virtualenv_root = os.path.dirname(os.path.dirname(sys.executable))
                self.config.read(os.path.join(virtualenv_root, self.config_path))

        if self.content:
            for section_id, section in self.content.items():
                if not isinstance(section, dict):
                    raise ValueError('Section descriptor is not a dict')

                for entry_id, entry in section.items():
                    if not isinstance(entry, django_configure.type.ConfigField):
                        raise ValueError('%s.%s - Entry is not inherited from django_configure.type.ConfigField' %
                                         (section_id, entry_id))
                    entry.init(env_prefix, section_id, entry_id, self.config)

    @property
    def config_var_name(self):
        return (self.env_prefix.upper() + '_' if self.env_prefix else '') + 'CONFIG'

    def get(self, descriptor):
        descriptor = descriptor.split('.')
        assert len(descriptor) == 2
        return self.content[descriptor[0]][descriptor[1]].get()

    def append(self, description_dict):
        if not self.content:
            self.content = description_dict
        else:
            for section_id, section in description_dict.items():
                if not isinstance(section, dict):
                    raise ValueError('Section descriptor is not a dict')

                if section_id not in self.content:
                    self.content[section] = {}

                for entry_id, entry in description_dict[section].items():
                    if not isinstance(entry, django_configure.type.ConfigField):
                        raise ValueError('%s.%s - Entry is not inherited from django_configure.type.ConfigField' %
                                         (section_id, entry_id))
                    entry.init(self.env_prefix, section, entry_id)
                    self.content[section][entry_id] = entry

    def generate(self, generate_file=True, dev=False):
        output = ''
        for section_id, section in self.content.items():
            output += '[%s]\n' % section_id
            for entry_id, entry in section.items():
                output += '# %s\n' % entry.help
                output += '%s=%s\n' % (entry.id, entry.dev_default if dev else entry.default)
        if self.config_path and generate_file:
            try:
                os.makedirs(os.path.dirname(self.config_path))
            except Exception:
                pass
            path = self.config_path
            if os.path.exists(path):
                path += '.template'

            with open(path, mode='w') as f:
                f.write(output)

        return output

    def generate_wsgi(self, path=None):
        wsgi_template = ''
        with open(os.path.join(os.path.dirname(__file__), 'resources', 'wsgi.py.template'), mode='r') as wsgi_template_file:
            wsgi_template = wsgi_template_file.read()

        new_wsgi_content = wsgi_template % {'CONFIG_VAR_NAME': self.config_var_name, 'CONFIG_PATH': self.config_path}

        if path is not None:
            with open(path, mode='w') as wsgi_file:
                wsgi_file.write(new_wsgi_content)

        return new_wsgi_content


def check_config(config):
    if config.config_var_name not in os.environ:
        print('[WARNING] %s env variable is not set, if you run any manage.py task apart from createconfig stop now, '
              'and set %s to your application config (generated via \'manage.py createconfig\'' %
              (config.config_var_name, config.config_var_name))


def define(description_dict, default_path=None, env_prefix=''):
    global main_config
    main_config = DjangoConfig(description_dict, env_prefix=env_prefix, path=default_path)
    check_config(main_config)
    return main_config


def default(app_name, base_path, env_prefix='', default_path=None):
    def generate_secret():
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        return get_random_string(50, chars)

    global main_config
    main_config = DjangoConfig({
        'Common': {
            'secret': django_configure.type.String('Secret for the application', default=generate_secret()),
            'static_root': django_configure.type.Path('Static root path (static files will be copied here)',
                                                      default='/var/lib/' + app_name + '/static/',
                                                      dev_default=os.path.join(base_path, 'static.root')),
            'static_url': django_configure.type.Url('Url to static files', default='/static/'),
            'media_root': django_configure.type.Path('Media root path (media files will be stored here)',
                                                     default='/var/lib/' + app_name + '/media/',
                                                     dev_default=os.path.join(base_path, 'media.root')),
            'media_url': django_configure.type.Url('Url to media files', '/media/'),
            'debug': django_configure.type.Boolean(help='if true debug mode will be enabled for the application, do not switch this in production',
                                                   default=False, dev_default=True)
        },
        'Database': {
            'url': django_configure.type.Database('Url to access database (including credentials)',
                                                  default='sqlite:////var/lib/'+app_name+'/'+app_name+'.sqlite',
                                                  dev_default='sqlite:///'+base_path+'/'+app_name+'.sqlite')
        }
    }, env_prefix=env_prefix, path=default_path)

    check_config(main_config)

    return main_config



import os

import dj_database_url


class ConfigField(object):
    def __init__(self, help='', default=None, dev_default=None):
        self.help = help
        self._default = default
        self._dev_default = dev_default or default
        self.id = None
        self.group = None
        self.env_prefix = ''
        self.config = None

    def init(self, env_prefix, group, id, config):
        self.env_prefix = env_prefix
        self.id = id
        self.group = group
        self.config = config

    @property
    def default(self):
        return self._default() if callable(self._default) else self._default

    @property
    def dev_default(self):
        return self._dev_default() if callable(self._dev_default) else self._dev_default or self.default

    def _convert(self, value):
        """This method should be overloaded in other types. value is always string
        (can come from either config file or environmental variable)

        :param value:
        :type value: str
        :return: Object
        :rtype: object
        """
        return value

    @property
    def env_variable(self):
        return ((self.env_prefix.upper() + '_') if self.env_prefix else '') + self.group.upper() + '_' + self.id.upper()

    def get(self):
        var = self._convert(os.environ.get(self.env_variable) or self.config.get(self.group, self.id))
        if var is None:
            print('[WARNING] (this is ok during config generation) No %s.%s in config file, '
                  'make sure that env variable %s was set to app config, and '
                  'it has all required fields defined, using default: %s' % (self.group, self.id,
                                                                             (self.env_prefix + '_' if self.env_prefix else '') + 'CONFIG',
                                                                             self.default))
            return self.default
        return var


class Url(ConfigField):
    """For now it's the same as default (String), in the future may be extended

    """
    pass


class Integer(ConfigField):
    def _convert(self, value):
        return int(value)


class Float(ConfigField):
    def _convert(self, value):
        return float(value)


class String(ConfigField):
    """For now it's the same as default (String), in the future may be extended

        """
    pass


class Database(ConfigField):
    def get(self):
        db_url = super(Database, self).get()
        return dj_database_url.parse(db_url, conn_max_age=600)


class Boolean(ConfigField):
    def _convert(self, value):
        if isinstance(value, Boolean):
            return value

        if isinstance(value, str):
            if value.lower() in ('true', '1'):
                return True
            elif value.lower() in ('false', 0):
                return False
            raise ValueError('%s is not a proper boolean' % value)

        return value is not None


class Path(ConfigField):
    def __init__(self, help='', default=None, dev_default=None, root='/', dev_root=None):
        super(Path, self).__init__(help, default, dev_default)
        self.root = root
        self.dev_root = dev_root

    @property
    def dev_default(self):
        path = super(Path, self).dev_default
        print 'get default %s' % path
        if not os.path.isabs(path):
            return os.path.join(self.dev_root, path)
        return path

    @property
    def default(self):
        path = super(Path, self).default
        if not os.path.isabs(path):
            return os.path.join(self.root, path)
        return path




import tempfile
import unittest

import shutil

import os
from django.utils.crypto import get_random_string

import django_configure
import django_configure.type


class ConfigTestCase(unittest.TestCase):
    def tearDown(self):
        super(ConfigTestCase, self).tearDown()

    def setUp(self):
        super(ConfigTestCase, self).setUp()
        self.tmp_dir = os.path.join(tempfile.gettempdir(), 'django-config-test')
        self.config_path = os.path.join(self.tmp_dir, 'testapp.cfg')
        try:
            shutil.rmtree(os.path.dirname(self.config_path))
        except OSError:
            pass

    def test_config_define(self):

        def generate_string():
            chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
            return get_random_string(50, chars)

        generated_secret_string = generate_string()

        config = django_configure.define(description_dict={
            'Common': {
                'secret': django_configure.type.String(default=generated_secret_string),
                'static_root': django_configure.type.String()
            }
        }, default_path=self.config_path)

        self.assertEqual(config.get('Common.secret'), generated_secret_string)
        self.assertEqual(config.get('Common.static_root'), None)

        config_output = config.generate(True)

        self.assertTrue(os.path.exists(self.config_path))
        with open(self.config_path, 'r') as f:
            self.assertMultiLineEqual(config_output, f.read())

    def test_default(self):
        config = django_configure.default(app_name='testapp')
        self.assertTrue(config.get('Common.secret'))

    def test_secret_generation(self):
        config_01 = django_configure.default(app_name='testapp')
        config_02 = django_configure.default(app_name='testapp')

        self.assertNotEqual(config_01.get('Common.secret'), config_02.get('Common.secret'))

    def test_not_overwrite(self):
        config_01 = django_configure.default(app_name='testapp', default_path=self.config_path)
        out_1 = config_01.generate(True)

        with open(self.config_path, 'r') as f:
            self.assertMultiLineEqual(out_1, f.read())

        config_02 = django_configure.default(app_name='testapp', default_path=self.config_path)
        out_2 = config_02.generate(True)

        with open(self.config_path+'.template', 'r') as f:
            self.assertMultiLineEqual(out_2, f.read())

    def test_wsgi(self):
        config_01 = django_configure.default(app_name='testapp', env_prefix='TESTAPP', default_path=self.config_path)

        try:
            os.makedirs(self.tmp_dir)
        except IOError:
            pass

        wsgi_path = os.path.join(self.tmp_dir, 'wsgi.py')
        wsgi_out = config_01.generate_wsgi(wsgi_path)

        with open(wsgi_path, 'r') as f:
            self.assertMultiLineEqual(wsgi_out, f.read())

        # run wsgi code to make env variable available
        exec(wsgi_out)

        self.assertEqual(os.environ[config_01.config_var_name], self.config_path)

    def test_db(self):
        config_01 = django_configure.define(description_dict={
            'Database': {
                'url': django_configure.type.Database()
            }
        })
        sqlite_path = self.tmp_dir+'/db.sqlite'
        os.environ['DATABASE_URL'] = 'sqlite:///'+sqlite_path
        db = config_01.get('Database.url')
        self.assertDictContainsSubset({'ENGINE': 'django.db.backends.sqlite3', 'NAME': sqlite_path}, db)

    def test_env(self):
        config = django_configure.default(app_name='testapp', env_prefix='TESTAPP')
        os.environ['TESTAPP_COMMON_SECRET'] = 'new_secret'
        self.assertEqual('new_secret', config.get('Common.secret'))

if __name__ == '__main__':
    unittest.main()



# coding=utf-8
from setuptools import setup, find_packages
import os

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(
    name='django-configure',
    version='0.2.4',
    packages=find_packages(),
    url='',
    license='MIT License',
    author='Michał Szostak',
    author_email='szostak.m.f@gmail.com',
    description='Utility for defining external django configs (for end user)',
    install_requires=[
        'dj-database-url>=0.4.2'
    ],
    package_data={
        'django_configure': ['resources/wsgi.py.template'],
    },
    long_description=README,
    keywords = ['configuration', 'django'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        # 'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
